import { Autocomplete, Box, TextField, Typography, Paper } from "@mui/material";
import { Create, useAutocomplete } from "@refinedev/mui";
import { useForm } from "@refinedev/react-hook-form";
import { Controller } from "react-hook-form";
import { cityStore } from "../../store/CityStore";
import React, { useState, useEffect } from "react";
import { TOKEN_KEY } from "../../authProvider";
import { observer } from "mobx-react-lite";
import Cookies from "js-cookie";
import { useLocation } from "react-router";

export const SightCreate = observer(() => {
  const [language, setLanguage] = useState(Cookies.get("lang") || "ru");
  // Состояния для предпросмотра
  const handleLanguageChange = (lang: string) => {
    setLanguage(lang);
    Cookies.set("lang", lang);
  };

  useEffect(() => {
    const lang = Cookies.get("lang")!;
    Cookies.set("lang", language);

    return () => {
      Cookies.set("lang", lang);
    };
  }, [language]);

  const {
    saveButtonProps,
    refineCore: { formLoading },
    register,
    control,
    watch,
    setValue,
    formState: { errors },
  } = useForm({
    refineCoreProps: {
      resource: "sight/",
    },
  });
  const { city_id } = cityStore;

  const [namePreview, setNamePreview] = useState("");
  const [coordinatesPreview, setCoordinatesPreview] = useState({
    latitude: "",
    longitude: "",
  });
  const [cityPreview, setCityPreview] = useState("");
  const [thumbnailPreview, setThumbnailPreview] = useState<string | null>(null);
  const [watermarkLUPreview, setWatermarkLUPreview] = useState<string | null>(
    null
  );
  const [watermarkRDPreview, setWatermarkRDPreview] = useState<string | null>(
    null
  );
  const [leftArticlePreview, setLeftArticlePreview] = useState("");
  const [previewArticlePreview, setPreviewArticlePreview] = useState("");

  const handleCoordinatesChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const [lat, lon] = e.target.value.split(",").map((s) => s.trim());
    setCoordinatesPreview({
      latitude: lat,
      longitude: lon,
    });
    setValue("latitude", lat);
    setValue("longitude", lon);
  };

  // Автокомплиты
  const { autocompleteProps: cityAutocompleteProps } = useAutocomplete({
    resource: "city",
    onSearch: (value) => [
      {
        field: "name",
        operator: "contains",
        value,
      },
    ],
    meta: {
      headers: {
        "Accept-Language": language,
      },
    },
  });

  const { autocompleteProps: mediaAutocompleteProps } = useAutocomplete({
    resource: "media",
    onSearch: (value) => [
      {
        field: "media_name",
        operator: "contains",
        value,
      },
    ],
  });

  const { autocompleteProps: articleAutocompleteProps } = useAutocomplete({
    resource: "article",
    onSearch: (value) => [
      {
        field: "heading",
        operator: "contains",
        value,
      },
    ],
  });

  // Следим за изменениями во всех полях
  const nameContent = watch("name");
  const addressContent = watch("address");
  const latitudeContent = watch("latitude");
  const longitudeContent = watch("longitude");
  const cityContent = watch("city_id");
  const thumbnailContent = watch("thumbnail");
  const watermarkLUContent = watch("watermark_lu");
  const watermarkRDContent = watch("watermark_rd");
  const leftArticleContent = watch("left_article");
  const previewArticleContent = watch("preview_article");

  // Обновляем состояния при изменении полей
  useEffect(() => {
    setNamePreview(nameContent || "");
  }, [nameContent]);

  useEffect(() => {
    setCoordinatesPreview({
      latitude: latitudeContent || "",
      longitude: longitudeContent || "",
    });
  }, [latitudeContent, longitudeContent]);

  useEffect(() => {
    const selectedCity = cityAutocompleteProps.options.find(
      (option) => option.id === cityContent
    );
    setCityPreview(selectedCity?.name || "");
  }, [cityContent, cityAutocompleteProps.options]);

  useEffect(() => {
    const selectedThumbnail = mediaAutocompleteProps.options.find(
      (option) => option.id === thumbnailContent
    );
    setThumbnailPreview(
      selectedThumbnail
        ? `${import.meta.env.VITE_KRBL_MEDIA}${
            selectedThumbnail.id
          }/download?token=${localStorage.getItem(TOKEN_KEY)}`
        : null
    );
  }, [thumbnailContent, mediaAutocompleteProps.options]);

  useEffect(() => {
    if (city_id) {
      setValue("city_id", +city_id);
    }
  }, [city_id, setValue]);

  useEffect(() => {
    const selectedWatermarkLU = mediaAutocompleteProps.options.find(
      (option) => option.id === watermarkLUContent
    );
    setWatermarkLUPreview(
      selectedWatermarkLU
        ? `${import.meta.env.VITE_KRBL_MEDIA}${
            selectedWatermarkLU.id
          }/download?token=${localStorage.getItem(TOKEN_KEY)}`
        : null
    );
  }, [watermarkLUContent, mediaAutocompleteProps.options]);

  useEffect(() => {
    const selectedWatermarkRD = mediaAutocompleteProps.options.find(
      (option) => option.id === watermarkRDContent
    );
    setWatermarkRDPreview(
      selectedWatermarkRD
        ? `${import.meta.env.VITE_KRBL_MEDIA}${
            selectedWatermarkRD.id
          }/download?token=${localStorage.getItem(TOKEN_KEY)}`
        : null
    );
  }, [watermarkRDContent, mediaAutocompleteProps.options]);

  useEffect(() => {
    const selectedLeftArticle = articleAutocompleteProps.options.find(
      (option) => option.id === leftArticleContent
    );
    setLeftArticlePreview(selectedLeftArticle?.heading || "");
  }, [leftArticleContent, articleAutocompleteProps.options]);

  useEffect(() => {
    const selectedPreviewArticle = articleAutocompleteProps.options.find(
      (option) => option.id === previewArticleContent
    );
    setPreviewArticlePreview(selectedPreviewArticle?.heading || "");
  }, [previewArticleContent, articleAutocompleteProps.options]);

  return (
    <Create isLoading={formLoading} saveButtonProps={saveButtonProps}>
      <Box sx={{ display: "flex", gap: 2 }}>
        <Box sx={{ display: "flex", flexDirection: "column", flex: 1, gap: 2 }}>
          {/* Форма создания */}
          <Box
            sx={{
              flex: 1,
              display: "flex",
              gap: 2,
            }}
          >
            <Box
              sx={{
                cursor: "pointer",
                flex: 1,
                display: "flex",
                justifyContent: "center",
                bgcolor: language === "ru" ? "primary.main" : "transparent",
                color: language === "ru" ? "white" : "inherit",
                borderRadius: 1,
                p: 1,
              }}
              onClick={() => handleLanguageChange("ru")}
            >
              RU
            </Box>
            <Box
              sx={{
                cursor: "pointer",
                flex: 1,
                display: "flex",
                justifyContent: "center",
                bgcolor: language === "en" ? "primary.main" : "transparent",
                color: language === "en" ? "white" : "inherit",
                borderRadius: 1,
                p: 1,
              }}
              onClick={() => handleLanguageChange("en")}
            >
              EN
            </Box>
            <Box
              sx={{
                cursor: "pointer",
                flex: 1,
                display: "flex",
                justifyContent: "center",
                bgcolor: language === "zh" ? "primary.main" : "transparent",
                color: language === "zh" ? "white" : "inherit",
                borderRadius: 1,
                p: 1,
              }}
              onClick={() => handleLanguageChange("zh")}
            >
              ZH
            </Box>
          </Box>

          <Box
            component="form"
            sx={{ flex: 1, display: "flex", flexDirection: "column" }}
            autoComplete="off"
          >
            <TextField
              {...register("name", {
                required: "Это поле является обязательным",
              })}
              error={!!(errors as any)?.name}
              helperText={(errors as any)?.name?.message}
              margin="normal"
              fullWidth
              InputLabelProps={{ shrink: true }}
              type="text"
              label={"Название *"}
              name="name"
            />

            <TextField
              value={`${coordinatesPreview.latitude}, ${coordinatesPreview.longitude}`}
              onChange={handleCoordinatesChange}
              error={!!(errors as any)?.latitude}
              helperText={(errors as any)?.latitude?.message}
              margin="normal"
              fullWidth
              InputLabelProps={{ shrink: true }}
              type="text"
              label={"Координаты *"}
            />
            <input
              type="hidden"
              {...register("longitude", {
                value: coordinatesPreview.longitude,
                required: "Это поле является обязательным",
                valueAsNumber: true,
              })}
            />
            <input
              type="hidden"
              {...register("latitude", {
                value: coordinatesPreview.latitude,
                required: "Это поле является обязательным",
                valueAsNumber: true,
              })}
            />

            <TextField
              {...register("address", {
                required: "Это поле является обязательным",
              })}
              error={!!(errors as any)?.address}
              helperText={(errors as any)?.address?.message}
              margin="normal"
              fullWidth
              InputLabelProps={{ shrink: true }}
              type="text"
              label={"Адрес *"}
              name="address"
            />

            <Controller
              control={control}
              name="city_id"
              rules={{ required: "Это поле является обязательным" }}
              render={({ field }) => (
                <Autocomplete
                  {...cityAutocompleteProps}
                  value={
                    cityAutocompleteProps.options.find(
                      (option) => option.id === field.value
                    ) || null
                  }
                  onChange={(_, value) => {
                    field.onChange(value?.id || "");
                  }}
                  getOptionLabel={(item) => {
                    return item ? item.name : "";
                  }}
                  isOptionEqualToValue={(option, value) => {
                    return option.id === value?.id;
                  }}
                  filterOptions={(options, { inputValue }) => {
                    return options.filter((option) =>
                      option.name
                        .toLowerCase()
                        .includes(inputValue.toLowerCase())
                    );
                  }}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Выберите город"
                      margin="normal"
                      variant="outlined"
                      error={!!errors.city_id}
                      helperText={(errors as any)?.city_id?.message}
                      required
                    />
                  )}
                />
              )}
            />

            <Controller
              control={control}
              name="thumbnail"
              defaultValue={null}
              render={({ field }) => (
                <Autocomplete
                  {...mediaAutocompleteProps}
                  value={
                    mediaAutocompleteProps.options.find(
                      (option) => option.id === field.value
                    ) || null
                  }
                  onChange={(_, value) => {
                    field.onChange(value?.id || "");
                  }}
                  getOptionLabel={(item) => {
                    return item ? item.media_name : "";
                  }}
                  isOptionEqualToValue={(option, value) => {
                    return option.id === value?.id;
                  }}
                  filterOptions={(options, { inputValue }) => {
                    return options.filter((option) =>
                      option.media_name
                        .toLowerCase()
                        .includes(inputValue.toLowerCase())
                    );
                  }}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Выберите обложку"
                      margin="normal"
                      variant="outlined"
                      error={!!errors.thumbnail}
                      helperText={(errors as any)?.thumbnail?.message}
                      required
                    />
                  )}
                />
              )}
            />

            <Controller
              control={control}
              name="watermark_lu"
              defaultValue={null}
              render={({ field }) => (
                <Autocomplete
                  {...mediaAutocompleteProps}
                  value={
                    mediaAutocompleteProps.options.find(
                      (option) => option.id === field.value
                    ) || null
                  }
                  onChange={(_, value) => {
                    field.onChange(value?.id || "");
                  }}
                  getOptionLabel={(item) => {
                    return item ? item.media_name : "";
                  }}
                  isOptionEqualToValue={(option, value) => {
                    return option.id === value?.id;
                  }}
                  filterOptions={(options, { inputValue }) => {
                    return options.filter((option) =>
                      option.media_name
                        .toLowerCase()
                        .includes(inputValue.toLowerCase())
                    );
                  }}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Выберите водный знак (Левый верх)"
                      margin="normal"
                      variant="outlined"
                      error={!!errors.watermark_lu}
                      helperText={(errors as any)?.watermark_lu?.message}
                    />
                  )}
                />
              )}
            />

            <Controller
              control={control}
              name="watermark_rd"
              defaultValue={null}
              render={({ field }) => (
                <Autocomplete
                  {...mediaAutocompleteProps}
                  value={
                    mediaAutocompleteProps.options.find(
                      (option) => option.id === field.value
                    ) || null
                  }
                  onChange={(_, value) => {
                    field.onChange(value?.id || "");
                  }}
                  getOptionLabel={(item) => {
                    return item ? item.media_name : "";
                  }}
                  isOptionEqualToValue={(option, value) => {
                    return option.id === value?.id;
                  }}
                  filterOptions={(options, { inputValue }) => {
                    return options.filter((option) =>
                      option.media_name
                        .toLowerCase()
                        .includes(inputValue.toLowerCase())
                    );
                  }}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Выберите водный знак (Правый верх)"
                      margin="normal"
                      variant="outlined"
                      error={!!errors.watermark_rd}
                      helperText={(errors as any)?.watermark_rd?.message}
                    />
                  )}
                />
              )}
            />

            <Controller
              control={control}
              name="left_article"
              defaultValue={null}
              render={({ field }) => (
                <Autocomplete
                  {...articleAutocompleteProps}
                  value={
                    articleAutocompleteProps.options.find(
                      (option) => option.id === field.value
                    ) || null
                  }
                  onChange={(_, value) => {
                    field.onChange(value?.id || "");
                  }}
                  getOptionLabel={(item) => {
                    return item ? item.heading : "";
                  }}
                  isOptionEqualToValue={(option, value) => {
                    return option.id === value?.id;
                  }}
                  filterOptions={(options, { inputValue }) => {
                    return options.filter((option) =>
                      option.heading
                        .toLowerCase()
                        .includes(inputValue.toLowerCase())
                    );
                  }}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Левая статья"
                      margin="normal"
                      variant="outlined"
                      error={!!errors.left_article}
                      helperText={(errors as any)?.left_article?.message}
                    />
                  )}
                />
              )}
            />

            <Controller
              control={control}
              name="preview_article"
              defaultValue={null}
              render={({ field }) => (
                <Autocomplete
                  {...articleAutocompleteProps}
                  value={
                    articleAutocompleteProps.options.find(
                      (option) => option.id === field.value
                    ) || null
                  }
                  onChange={(_, value) => {
                    field.onChange(value?.id || "");
                  }}
                  getOptionLabel={(item) => {
                    return item ? item.heading : "";
                  }}
                  isOptionEqualToValue={(option, value) => {
                    return option.id === value?.id;
                  }}
                  filterOptions={(options, { inputValue }) => {
                    return options.filter((option) =>
                      option.heading
                        .toLowerCase()
                        .includes(inputValue.toLowerCase())
                    );
                  }}
                  renderInput={(params) => (
                    <TextField
                      {...params}
                      label="Cтатья-предпросмотр"
                      margin="normal"
                      variant="outlined"
                      error={!!errors.preview_article}
                      helperText={(errors as any)?.preview_article?.message}
                    />
                  )}
                />
              )}
            />
          </Box>
        </Box>

        {/* Preview Panel */}
        <Paper
          sx={{
            flex: 1,
            p: 2,
            maxHeight: "calc(100vh - 200px)",
            overflowY: "auto",
            position: "sticky",
            top: 16,
            borderRadius: 2,
            border: "1px solid",
            borderColor: "primary.main",
            bgcolor: (theme) =>
              theme.palette.mode === "dark" ? "background.paper" : "#fff",
          }}
        >
          <Typography variant="h6" gutterBottom color="primary">
            Предпросмотр
          </Typography>

          {/* Название достопримечательности */}
          <Typography
            variant="h4"
            gutterBottom
            sx={{
              color: (theme) =>
                theme.palette.mode === "dark" ? "grey.300" : "grey.800",
              mb: 3,
            }}
          >
            {namePreview}
          </Typography>

          {/* Город */}
          <Typography variant="body1" sx={{ mb: 2 }}>
            <Box component="span" sx={{ color: "text.secondary" }}>
              Город:{" "}
            </Box>
            <Box
              component="span"
              sx={{
                color: (theme) =>
                  theme.palette.mode === "dark" ? "grey.300" : "grey.800",
              }}
            >
              {cityPreview}
            </Box>
          </Typography>

          {/* Адрес */}
          <Typography variant="body1" sx={{ mb: 2 }}>
            <Box component="span" sx={{ color: "text.secondary" }}>
              Адрес:{" "}
            </Box>
            <Box
              component="span"
              sx={{
                color: (theme) =>
                  theme.palette.mode === "dark" ? "grey.300" : "grey.800",
              }}
            >
              {addressContent}
            </Box>
          </Typography>

          {/* Координаты */}
          <Typography variant="body1" sx={{ mb: 2 }}>
            <Box component="span" sx={{ color: "text.secondary" }}>
              Координаты:{" "}
            </Box>
            <Box
              component="span"
              sx={{
                color: (theme) =>
                  theme.palette.mode === "dark" ? "grey.300" : "grey.800",
              }}
            >
              {`${coordinatesPreview.latitude}, ${coordinatesPreview.longitude}`}
            </Box>
          </Typography>

          {/* Обложка */}
          {thumbnailPreview && (
            <Box sx={{ mb: 2 }}>
              <Typography
                variant="body1"
                gutterBottom
                sx={{ color: "text.secondary" }}
              >
                Обложка:
              </Typography>
              <Box
                component="img"
                src={thumbnailPreview}
                alt="Обложка"
                sx={{
                  maxWidth: "100%",
                  height: "40vh",
                  borderRadius: 2,
                  border: "1px solid",
                  borderColor: "primary.main",
                }}
              />
            </Box>
          )}

          {/* Водяные знаки */}
          <Box sx={{ mb: 2 }}>
            <Typography
              variant="body1"
              gutterBottom
              sx={{ color: "text.secondary" }}
            >
              Водяные знаки:
            </Typography>
            <Box sx={{ display: "flex", gap: 2 }}>
              {watermarkLUPreview && (
                <Box>
                  <Typography
                    variant="body2"
                    gutterBottom
                    sx={{ color: "text.secondary" }}
                  >
                    Левый верхний:
                  </Typography>
                  <Box
                    component="img"
                    src={watermarkLUPreview}
                    alt="Водяной знак (ЛВ)"
                    sx={{
                      width: 100,
                      height: 100,
                      objectFit: "cover",
                      borderRadius: 1,
                      border: "1px solid",
                      borderColor: "primary.main",
                    }}
                  />
                </Box>
              )}
              {watermarkRDPreview && (
                <Box>
                  <Typography
                    variant="body2"
                    gutterBottom
                    sx={{ color: "text.secondary" }}
                  >
                    Правый верхний:
                  </Typography>
                  <Box
                    component="img"
                    src={watermarkRDPreview}
                    alt="Водяной знак (ПН)"
                    sx={{
                      width: 100,
                      height: 100,
                      objectFit: "cover",
                      borderRadius: 1,
                      border: "1px solid",
                      borderColor: "primary.main",
                    }}
                  />
                </Box>
              )}
            </Box>
          </Box>

          {/* Связанные статьи */}
          <Box>
            {leftArticlePreview && (
              <Typography variant="body1" gutterBottom>
                <Box component="span" sx={{ color: "text.secondary" }}>
                  Левая статья:{" "}
                </Box>
                <Box
                  component="span"
                  sx={{
                    color: (theme) =>
                      theme.palette.mode === "dark" ? "grey.300" : "grey.800",
                  }}
                >
                  {leftArticlePreview}
                </Box>
              </Typography>
            )}
            {previewArticlePreview && (
              <Typography variant="body1" gutterBottom>
                <Box component="span" sx={{ color: "text.secondary" }}>
                  Статья-предпросмотр:{" "}
                </Box>
                <Box
                  component="span"
                  sx={{
                    color: (theme) =>
                      theme.palette.mode === "dark" ? "grey.300" : "grey.800",
                  }}
                >
                  {previewArticlePreview}
                </Box>
              </Typography>
            )}
          </Box>
        </Paper>
      </Box>
    </Create>
  );
});
