import {
  Autocomplete,
  Box,
  TextField,
  Typography,
  FormControlLabel,
  Paper,
  Grid,
  Checkbox,
} from "@mui/material";
import { Edit, useAutocomplete } from "@refinedev/mui";
import { useForm } from "@refinedev/react-hook-form";
import { Controller } from "react-hook-form";

import { useParams } from "react-router";
import { LinkedItems } from "../../components/LinkedItems";
import { type SightItem, sightFields } from "./types";

const TRANSFER_FIELDS = [
  { name: "bus", label: "Автобус" },
  { name: "metro_blue", label: "Метро (синяя)" },
  { name: "metro_green", label: "Метро (зеленая)" },
  { name: "metro_orange", label: "Метро (оранжевая)" },
  { name: "metro_purple", label: "Метро (фиолетовая)" },
  { name: "metro_red", label: "Метро (красная)" },
  { name: "train", label: "Электричка" },
  { name: "tram", label: "Трамвай" },
  { name: "trolleybus", label: "Троллейбус" },
];

export const StationEdit = () => {
  const {
    saveButtonProps,
    register,
    control,
    formState: { errors },
  } = useForm({});

  const { id: stationId } = useParams<{ id: string }>();

  const { autocompleteProps: cityAutocompleteProps } = useAutocomplete({
    resource: "city",
    onSearch: (value) => [
      {
        field: "name",
        operator: "contains",
        value,
      },
    ],
  });

  return (
    <Edit saveButtonProps={saveButtonProps}>
      <Box
        component="form"
        sx={{ display: "flex", flexDirection: "column" }}
        autoComplete="off"
      >
        <TextField
          {...register("name", {
            required: "Это поле является обязательным",
          })}
          error={!!(errors as any)?.name}
          helperText={(errors as any)?.name?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Название *"}
          name="name"
        />
        <TextField
          {...register("system_name", {
            required: "Это поле является обязательным",
          })}
          error={!!(errors as any)?.system_name}
          helperText={(errors as any)?.system_name?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Системное название *"}
          name="system_name"
        />
        <Controller
          name="direction" // boolean
          control={control}
          defaultValue={false}
          render={({ field }: { field: any }) => (
            <FormControlLabel
              label="Прямой маршрут?"
              control={
                <Checkbox
                  {...field}
                  checked={field.value}
                  onChange={(e) => field.onChange(e.target.checked)}
                />
              }
            />
          )}
        />
        <TextField
          {...register("description", {
            // required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.description}
          helperText={(errors as any)?.description?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Описание"}
          name="description"
        />
        <TextField
          {...register("address", {
            // required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.address}
          helperText={(errors as any)?.address?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="text"
          label={"Адрес"}
          name="address"
        />
        <TextField
          {...register("latitude", {
            required: "Это поле является обязательным",
            valueAsNumber: true,
          })}
          error={!!(errors as any)?.latitude}
          helperText={(errors as any)?.latitude?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="number"
          label={"Широта *"}
          name="latitude"
        />
        <TextField
          {...register("longitude", {
            required: "Это поле является обязательным",
            valueAsNumber: true,
          })}
          error={!!(errors as any)?.longitude}
          helperText={(errors as any)?.longitude?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="number"
          label={"Долгота *"}
          name="longitude"
        />

        <Controller
          control={control}
          name="city_id"
          rules={{ required: "Это поле является обязательным" }}
          defaultValue={null}
          render={({ field }) => (
            <Autocomplete
              {...cityAutocompleteProps}
              value={
                cityAutocompleteProps.options.find(
                  (option) => option.id === field.value
                ) || null
              }
              onChange={(_, value) => {
                field.onChange(value?.id || "");
              }}
              getOptionLabel={(item) => {
                return item ? item.name : "";
              }}
              isOptionEqualToValue={(option, value) => {
                return option.id === value?.id;
              }}
              filterOptions={(options, { inputValue }) => {
                return options.filter((option) =>
                  option.name.toLowerCase().includes(inputValue.toLowerCase())
                );
              }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Выберите город"
                  margin="normal"
                  variant="outlined"
                  error={!!errors.city_id}
                  helperText={(errors as any)?.city_id?.message}
                  required
                />
              )}
            />
          )}
        />

        <TextField
          {...register("offset_x", {
            // required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.offset_x}
          helperText={(errors as any)?.offset_x?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="number"
          label={"Смещение (X)"}
          name="offset_x"
        />

        <TextField
          {...register("offset_y", {
            // required: 'Это поле является обязательным',
          })}
          error={!!(errors as any)?.offset_y}
          helperText={(errors as any)?.offset_y?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{ shrink: true }}
          type="number"
          label={"Смещение (Y)"}
          name="offset_y"
        />

        {/* Группа полей пересадок */}
        <Paper sx={{ p: 2, mt: 2 }}>
          <Typography variant="h6" gutterBottom>
            Пересадки
          </Typography>
          <Grid container spacing={2}>
            {TRANSFER_FIELDS.map((field) => (
              <Grid item xs={12} sm={6} md={4} key={field.name}>
                <TextField
                  {...register(`transfers.${field.name}`)}
                  error={!!(errors as any)?.transfers?.[field.name]}
                  helperText={(errors as any)?.transfers?.[field.name]?.message}
                  margin="normal"
                  fullWidth
                  InputLabelProps={{ shrink: true }}
                  type="text"
                  label={field.label}
                  name={`transfers.${field.name}`}
                />
              </Grid>
            ))}
          </Grid>
        </Paper>
      </Box>

      {stationId && (
        <LinkedItems<SightItem>
          type="edit" // display and manage
          parentId={stationId}
          parentResource="station"
          childResource="sight"
          fields={sightFields}
          title="достопримечательности"
          dragAllowed={false}
        />
      )}
    </Edit>
  );
};
