import React, {
  createContext,
  useState,
  useContext,
  ReactNode,
  useMemo,
  useEffect,
} from "react";
import { geoMercator } from "d3-geo";
import { Coordinates, Track, uuid } from "@mt/common-types";
import { useNearStation, usePassedTrackIndex } from "./hooks";
import { AttractionGroup, MapData, StationOnMap } from "./map-widget.interface";
import { getMapPoint } from "./utils";
import { EMPTY_SETTING_VALUE, zeroCoordinates } from "./map-widget.constant";
import {
  MapSettings,
  MapWidgetContextType,
} from "./map-widget-context.interface";
import { useForm } from "react-hook-form";

export const mapCanvasProps = {
  style: {
    width: "100%",
    height: "100%",
  },
  width: 500,
  height: 400,
};

// prettier-ignore
export const MapWidgetContext = createContext<MapWidgetContextType | undefined>(undefined);

export const MapWidgetProvider = ({ children }: { children: ReactNode }) => {
  const [track, setTrack] = useState<Track | null>(null);
  const [stations, setStations] = useState<StationOnMap[]>([]);
  const [updatedStationIds, setUpdatedStationIds] = useState<uuid[]>([]);
  const [attractionGroups, setAttractionGroups] = useState<AttractionGroup[]>(
    []
  );
  const [rotateAngle, setRotateAngle] = useState<number>(0);

  const [scale, setScale] = useState<number>(0);
  const [fullScale, setFullScale] = useState<number>(0);
  const [zoomedScale, setZoomedScale] = useState<number>(0);

  const [center, setCenter] = useState(zeroCoordinates);
  const [baseCenter, setBaseCenter] = useState(zeroCoordinates);

  const [currentPosition, setCurrentPosition] = useState<Coordinates | null>(
    null
  );

  const [isEditMode, setIsEditMode] = useState<boolean>(false);
  const [isDragMode, setIsDragMode] = useState<boolean>(false);

  const [initialSettingsData, setInitialSettingsData] =
    useState<MapSettings>(EMPTY_SETTING_VALUE);
  const [isSettingsDataChanged, setIsSettingsDataChanged] =
    useState<boolean>(false);

  const isMapDataChanged = useMemo(
    () => isSettingsDataChanged || updatedStationIds.length > 0,
    [isSettingsDataChanged, updatedStationIds]
  );

  const stationsMap = useMemo(
    () => new Map(stations.map((station) => [station.id, station])),
    [stations]
  );

  const middleTrackCoordinates: Coordinates | null = useMemo(() => {
    if (!track?.length) {
      return null;
    }

    const middleTrackIndex = Math.floor(track.length / 2);

    return track[middleTrackIndex];
  }, [track]);

  const settingsForm = useForm({
    mode: "onChange",
    reValidateMode: "onChange",
    defaultValues: initialSettingsData,
  });
  useEffect(
    () => settingsForm.reset(initialSettingsData),
    [initialSettingsData]
  );

  const onMapDataFetched = (data: MapData) => {
    setTrack(data.trackPoints);
    setStations(data.stationsOnMap);
    setAttractionGroups(data.touristAttractionGroupsOnMap);
    setRotateAngle(data.mapRotateAngle);

    setCenter(data.centerOfMapPoint);
    setBaseCenter(data.centerOfMapPoint);

    setScale(data.fullMapScale);
    setFullScale(data.fullMapScale);
    setZoomedScale(data.zoomedMapScale);

    setInitialSettingsData({
      rotateAngle: data.mapRotateAngle,
      center: data.centerOfMapPoint,
      fullScale: data.fullMapScale,
      zoomedScale: data.zoomedMapScale,
    });

    setIsSettingsDataChanged(false);
    setUpdatedStationIds([]);
  };

  const onSettingsFormChange = () => {
    const formData = settingsForm.getValues();
    const { center, rotateAngle, fullScale, zoomedScale, currentStationId } =
      formData;

    setBaseCenter(center);
    setRotateAngle(rotateAngle);
    setFullScale(fullScale);
    setZoomedScale(zoomedScale);

    if (currentStationId) {
      const { pointOnMap } = stationsMap.get(currentStationId) as StationOnMap;
      setCenter(pointOnMap);
      setScale(zoomedScale);
      setCurrentPosition(pointOnMap);
      setIsDragMode(false);
    } else {
      setCenter(center);
      setScale(fullScale);
      setCurrentPosition(middleTrackCoordinates);
    }

    updateMapDataChanged(formData);
  };

  const onMapCenterMoved = (center: Coordinates) => {
    setBaseCenter(center);
    setCenter(center);
    settingsForm.setValue("center", center, { shouldDirty: true });

    updateMapDataChanged(settingsForm.getValues());
  };

  const updateMapDataChanged = (data: MapSettings) => {
    const { rotateAngle, center, fullScale, zoomedScale } = data;

    setIsSettingsDataChanged(
      JSON.stringify({
        rotateAngle,
        center,
        fullScale,
        zoomedScale,
      }) !== JSON.stringify(initialSettingsData)
    );
  };

  const onStationUpdate: MapWidgetContextType["onStationUpdate"] = (
    stationId,
    { labelOffset, labelAlignment }
  ) => {
    const updatedStation = {
      ...(stationsMap.get(stationId) as StationOnMap),
      ...(labelOffset && { labelOffset }),
      ...(labelAlignment && { labelAlignment }),
    };

    setStations((stations) =>
      stations.map((station) =>
        station.id === stationId ? updatedStation : station
      )
    );
    setUpdatedStationIds((ids) => [...ids, stationId]);
  };

  const getUpdatedStations = () => {
    return updatedStationIds.reduce((acc: Record<uuid, any>, id: uuid) => {
      const { labelAlignment, labelOffset } = stationsMap.get(
        id
      ) as StationOnMap;

      acc[id] = {
        textAlignment: labelAlignment,
        mapOffsets: labelOffset,
      };

      return acc;
    }, {});
  };

  const projection = useMemo(() => {
    const { width, height } = mapCanvasProps;

    return geoMercator()
      .translate([width / 2, height / 2])
      .center(getMapPoint(center))
      .scale(scale);
  }, [center, scale]);

  const { passedTrackIndex } = usePassedTrackIndex(track, currentPosition);
  const { currentStation, nextStation, isOnStation } = useNearStation(
    currentPosition,
    stations,
    passedTrackIndex
  );

  // Bind map center and zoom to currentStation in not EditMode
  useEffect(() => {
    if (isEditMode) {
      return;
    }

    if (currentStation) {
      const { pointOnMap } = currentStation;
      setCenter(pointOnMap);
      setScale(zoomedScale);
    } else {
      setCenter(baseCenter);
      setScale(fullScale);
    }
  }, [currentStation]);

  const contextValue = {
    track,
    center,
    rotateAngle,
    projection,
    attractionGroups,
    stations,
    currentPosition,
    middleTrackCoordinates,
    passedTrackIndex,
    currentStation,
    isOnStation,
    nextStation,

    setCurrentPosition,

    isDragMode,
    setIsDragMode,
    isEditMode,
    setIsEditMode,

    onMapDataFetched,
    settingsForm,
    onSettingsFormChange,
    onMapCenterMoved,
    onStationUpdate,
    getUpdatedStations,

    isMapDataChanged,
  };

  return (
    <MapWidgetContext.Provider value={contextValue}>
      {children}
    </MapWidgetContext.Provider>
  );
};

export const useMapWidgetContext = function (): MapWidgetContextType {
  const context = useContext(MapWidgetContext);

  if (!context) {
    throw new Error(
      "useMapWidgetContext must be used within a MapWidgetProvider"
    );
  }

  return context;
};
