import { TextField } from '@mui/material';
import { Controller, useFormContext } from 'react-hook-form';
import { ChangeEvent, useCallback } from 'react';
import { Coordinates } from '@mt/common-types';
import { FieldProps } from './field-props.interface';

interface CoordinateFieldProps extends FieldProps {
  onChange?: (event: ChangeEvent) => void;
}

const COORDINATE_PATTERN = /^(-?\d+(\.\d+)?),(\s*)(-?\d+(\.\d+)?)$/;
const formatValue = (value: string | Coordinates) =>
  typeof value === 'string' ? value : `${value.lat}, ${value.lon}`;
export const CoordinateField = ({ name, label, hint, onChange }: CoordinateFieldProps) => {
  const { getFieldState, control, setError, clearErrors } = useFormContext();

  const handleChange = useCallback(
    ({ target }, field) => {
      const { value } = target;

      let newValue: string | { lat: number; lon: number } = value;

      if (COORDINATE_PATTERN.test(value)) {
        const matches = value.match(COORDINATE_PATTERN) || [];
        const [lat, lon] = (matches[0] ?? '').split(',');

        newValue = { lat: Number(lat?.trim()), lon: Number(lon?.trim()) };
        clearErrors();
      } else {
        setError(field.name, {
          message: 'Неверный формат координаты',
        });
      }

      return field.onChange(newValue);
    },
    [clearErrors, setError]
  );

  return (
    <Controller
      name={name}
      control={control}
      render={({ field }) => (
        <TextField
          type="text"
          label={label}
          value={formatValue(field.value)}
          onChange={(e) => {
            onChange?.(e);
            handleChange(e, field);
          }}
          error={getFieldState(field.name).invalid}
          helperText={getFieldState(field.name).error?.message || hint || ''}
        />
      )}
    />
  );
};
