import { useRef, useState } from "react";
import { useLoading } from "../loader";
import { useQuery, QueryObserverResult } from "@tanstack/react-query";

type QueryFnData<TData> = () => Promise<TData>;

export interface LongPollingQueryOptions<TData> {
  pollingInterval?: number;
  maxRetries?: number;
}

export const useLongPollingQuery = <TData, TError = unknown>(
  queryKey: readonly unknown[],
  queryFn: QueryFnData<TData>,
  options?: LongPollingQueryOptions<TData>
): QueryObserverResult<TData, TError> => {
  const { pollingInterval = 5000, maxRetries = 5 } = options || {};
  const retriesCount = useRef<number>(0);
  const [refetchInterval, setRefetchInterval] = useState(pollingInterval);
  const { isLoading, showLoadingScreen } = useLoading();

  const query = useQuery<TData, TError, TData>({
    queryKey,
    queryFn,
    refetchInterval,
    refetchOnWindowFocus: false,
    retry: (failureCount, error) => {
      if (failureCount < maxRetries) {
        return true;
      } else {
        setRefetchInterval(0);
        if (!isLoading) showLoadingScreen();
        return false;
      }
    },
  });

  if (query.status === "success") {
    retriesCount.current = 0;
    setRefetchInterval(pollingInterval);
  }

  return query;
};
