import { useServerLocalization } from "@mt/i18n";

import React, { useCallback, useEffect, useRef, useState } from "react";
import "./AllAttractionsDropdown.css";
import { FormattedMessage } from "react-intl";

import { useGetAttractionList } from "./useGetAttractionList";
import { Icons, TouchScrollWrapper } from "@mt/components";

interface AllAttractionsDropdownProps {
  isIdleMode: boolean;
  handleAttractionClick: (id: string) => void;
}

const AllAttractionsDropdown = React.memo(
  ({ isIdleMode, handleAttractionClick }: AllAttractionsDropdownProps) => {
    const { data: attractions = [] } = useGetAttractionList();

    const [isOpen, setIsOpen] = useState(false);
    const attractionsListRef = useRef<HTMLUListElement>(null);
    const localizeText = useServerLocalization();
    const letters = attractions
      .sort((a, b) =>
        localizeText(a.name)
          .toLowerCase()
          .localeCompare(localizeText(b.name).toLowerCase())
      )
      .reduce((acc: string[], cur) => {
        const firstLetter = localizeText(cur.name).charAt(0).toUpperCase();

        if (!acc.includes(firstLetter)) {
          return acc.concat(firstLetter);
        }

        return acc;
      }, []);

    const handleLetterClick = useCallback(
      (letter: string) => {
        const attractionByFirstLetter = attractions.find((it) =>
          localizeText(it.name).startsWith(letter)
        );

        if (attractionByFirstLetter) {
          const item = attractionsListRef.current?.querySelector(
            `#attraction-${attractionByFirstLetter.id}`
          );

          item?.scrollIntoView({
            behavior: "smooth",
            block: "end",
            inline: "nearest",
          });
        }
      },
      [attractions, localizeText]
    );

    useEffect(() => {
      if (isIdleMode) setIsOpen(false);
    }, [isIdleMode]);

    return (
      <div className="dropdown">
        <button
          className="dropdown-button"
          onPointerUp={() => setIsOpen(!isOpen)}
        >
          <Icons.ChevronIcon className="dropdown-icon" />

          <div className="g-flex__item">
            <FormattedMessage id="attractions" />
          </div>
        </button>

        {isOpen && (
          <div className="dropdown-content">
            <div
              className="dropdown-content-header"
              onPointerUp={() => setIsOpen(!isOpen)}
            >
              <Icons.ChevronIcon className="dropdown-icon dropdown-icon--opened" />

              <div className="g-flex__item">
                <FormattedMessage id="attractions" />
              </div>
            </div>

            <TouchScrollWrapper className="dropdown-content-list">
              <ul className="attraction-list" ref={attractionsListRef}>
                {attractions.map((attraction) => (
                  <li
                    id={`attraction-${attraction.id}`}
                    key={attraction.id}
                    className="dropdown-list-item"
                    onPointerUp={() => {
                      setIsOpen(!isOpen);
                      handleAttractionClick(attraction.id);
                    }}
                  >
                    <div className="dropdown-list-item__image-wrapper">
                      <img
                        src={attraction.iconUrl}
                        alt={localizeText(attraction.name)}
                      />
                    </div>
                    <span>{localizeText(attraction.name)}</span>
                  </li>
                ))}
              </ul>
            </TouchScrollWrapper>

            <ul className="alphabet-list">
              {letters.map((letter) => (
                <li key={letter} onPointerUp={() => handleLetterClick(letter)}>
                  <span>{letter}</span>
                </li>
              ))}
            </ul>
          </div>
        )}
      </div>
    );
  }
);

export default AllAttractionsDropdown;
