import { useState, useEffect, useRef, useCallback } from "react";
import { AttractionWidget } from "@mt/components";
import AllAttractionsDropdown from "../all-attractions-dropdown/all-attractions-dropdown";
import "./AttractionsWidgetContainer.css";
import { useLightboxContext } from "@mt/components";
import {
  Article,
  ArticleBase,
  PhotoSphereLightboxData,
} from "@mt/common-types";

const POLL_INTERVAL_MS = 5000;

const IDLE_TIME_MS = 2 * 60 * 1000; // Idle time in milliseconds before restarting polling (2 minutes)
// const IDLE_TIME_MS = 15 * 1000; // debug

export function AttractionWidgetContainer() {
  const [touristArticles, setTouristArticles] = useState<ArticleBase[] | null>(
    null
  );
  const [error, setError] = useState<TypeError | null>(null);
  const [isPolling, setIsPolling] = useState<boolean>(true);
  const [isIdleMode, setIdleMode] = useState<boolean>(true);
  const idleTimeoutRef = useRef<NodeJS.Timeout | null>(null);
  const containerRef = useRef<HTMLDivElement>(null);
  const pollTimeoutRef = useRef<NodeJS.Timeout | null>(null);
  const abortControllerRef = useRef<AbortController | null>(null);

  const { closeLightbox } = useLightboxContext<PhotoSphereLightboxData>();

  const startPolling = useCallback(async () => {
    closeLightbox();
    setIsPolling(true);

    abortControllerRef.current = new AbortController();
    const poll = async (attempt = 1) => {
      try {
        const response = await fetch(
          "https://localhost:8443/widgets/attraction-details/events",
          {
            signal: abortControllerRef.current?.signal,
          }
        ).then((resp) => resp.json());

        // if we receive empty array - preserve previous attraction on the screen
        if (response?.[0]) {
          const [{ touristAttractionInfoPages }] = response;

          const articles: ArticleBase[] = touristAttractionInfoPages.map(
            ({ title: name, ...rest }: Article) => ({ name, ...rest })
          );

          setTouristArticles(articles);
        }

        setError(null);

        if (isPolling) {
          // Schedule the next poll
          pollTimeoutRef.current = setTimeout(poll, POLL_INTERVAL_MS);
        }
      } catch (error: unknown) {
        if ((error as TypeError).name === "AbortError") {
          console.log("Request was cancelled");
        } else {
          setError(error as TypeError);

          if (isPolling && attempt < 3) {
            // Schedule the next poll
            pollTimeoutRef.current = setTimeout(
              () => poll(attempt + 1),
              POLL_INTERVAL_MS
            );
          }
        }
      }
    };

    await poll();

    return () => {
      abortControllerRef.current?.abort();
    };
  }, [isPolling]);

  const stopPolling = () => {
    setIsPolling(false);
    abortControllerRef.current?.abort();

    if (pollTimeoutRef.current) {
      clearTimeout(pollTimeoutRef.current);
    }
  };

  const handleUserAction = (e: Event) => {
    const widgetContainer = containerRef.current;

    if (!widgetContainer || !e.target) return;
    if (!widgetContainer.contains(e.target as Node)) return;

    e.stopPropagation();
    resetIdleMode();
  };

  const handleAttractionSelect = useCallback(async (id: string) => {
    const { touristAttractionInfoPages } = await fetch(
      "https://localhost:8443/widgets/attraction-details/data-by-params?" +
        new URLSearchParams({ touristAttractionId: id })
    ).then((resp) => resp.json());

    resetIdleMode();

    const articles: ArticleBase[] = touristAttractionInfoPages.map(
      ({ title: name, ...rest }: Article) => ({ name, ...rest })
    );

    setTouristArticles(articles);
    // https://tracker.yandex.ru/TS-32
    document.querySelector(".widget-text.active")!.scrollTop = 0;
  }, []);

  const resetIdleMode = () => {
    abortControllerRef.current?.abort(); // cancel ongoing requests
    idleTimeoutRef.current && clearTimeout(idleTimeoutRef.current); // reset idle timeout
    setIdleMode(false);

    idleTimeoutRef.current = setTimeout(() => setIdleMode(true), IDLE_TIME_MS);
  };

  useEffect(() => {
    isIdleMode ? startPolling() : stopPolling();

    return stopPolling;
  }, [isIdleMode, startPolling]);

  // Attach event listeners to stop polling on user actions
  useEffect(() => {
    window.addEventListener("click", handleUserAction);
    window.addEventListener("scroll", handleUserAction);

    return () => {
      window.removeEventListener("click", handleUserAction);
      window.removeEventListener("scroll", handleUserAction);

      if (idleTimeoutRef.current) {
        clearTimeout(idleTimeoutRef.current);
      }
    };
  }, [containerRef]);

  return error ? (
    <p>Error: {error.message}</p>
  ) : (
    touristArticles && (
      <div
        className="attractions-widget-container g-flex-column g-flex--justify-end"
        ref={containerRef}
      >
        <AttractionWidget articles={touristArticles} isIdleMode={isIdleMode} />

        <AllAttractionsDropdown
          isIdleMode={isIdleMode}
          handleAttractionClick={handleAttractionSelect}
        />
      </div>
    )
  );
}
