import cn from 'classnames';
import { HTMLAttributes, useCallback, useId, useMemo, useState } from 'react';
import { FormattedMessage } from 'react-intl';
import { TouchScrollWrapper, TransportIcon } from '@mt/components';
import { uuid } from '@mt/common-types';
import { useServerLocalization } from '@mt/i18n';

import { Attraction, Station } from '../../nav-widget.interface';
import { NavAccordion, NavAccordionSummary, NavAccordionDetails } from '../../nav-widget.styles';
import { getDistanceFormatter } from '../get-distance-formatter';

const simulateClick = (id: uuid) => {
  document.getElementById(`accordSummary_${id}`)?.click();
};

export type NestedItems = Array<
  Attraction & Omit<Station, 'type'> & Partial<Pick<Station, 'type'>>
>;
export interface AccordionItem
  extends Omit<Station | Attraction, 'nearbyTouristAttractions' | 'nearbyTransportStops'> {
  nestedItems: NestedItems;
}

interface Props extends HTMLAttributes<HTMLDivElement> {
  titleId: string;
  isOpened: boolean;
  items: AccordionItem[];
  onExpandChange?: (isExpanded: boolean, itemId: uuid) => void;
  onNestedItemClick?: (itemId: uuid) => void;
}

export const AccordionListTab = (props: Props) => {
  const { titleId, isOpened, items, onExpandChange, onNestedItemClick, ...args } = props;
  const localizeText = useServerLocalization();
  const formatDistance = getDistanceFormatter(localizeText);
  const [expandedItem, setExpandedItem] = useState<uuid | null>(null);
  const scrollToExpanded = useCallback((id: uuid) => {
    const expandedEl = document.getElementById(`accordWrapper_${id}`);

    expandedEl?.scrollIntoView({
      behavior: 'smooth',
      block: 'nearest',
      inline: 'nearest',
    });
  }, []);

  const tabClasses = useMemo(
    () =>
      cn('g-flex-column__item-fixed g-flex-column tab-container', {
        'tab-container--opened': isOpened,
      }),
    [isOpened]
  );

  const isExpanded = useCallback((id: uuid) => id === expandedItem, [expandedItem]);

  const handleChange = (isExpanded: boolean, itemId: uuid) => {
    setExpandedItem(isExpanded ? itemId : null);
    onExpandChange?.(isExpanded, itemId);
  };

  return (
    <div className={tabClasses} {...args}>
      <div className="g-flex-column__item-fixed tab-title">
        <FormattedMessage id={titleId} />
      </div>

      <TouchScrollWrapper className="g-flex-column__item">
        {items.map(({ id, name, nestedItems }) => (
          <div className="tab-list__item" id={`accordWrapper_${id}`} key={id}>
            <div className="tab-list__item-expander" onPointerUp={() => simulateClick(id)} />

            <NavAccordion
              onTransitionEnd={() => isExpanded(id) && scrollToExpanded(id)}
              disabled={!nestedItems.length}
              expanded={isExpanded(id)}
              onChange={(_, expanded) => handleChange(expanded, id)}
            >
              <NavAccordionSummary id={`accordSummary_${id}`}>
                <div className="tab-list__item-label">{localizeText(name)}</div>
              </NavAccordionSummary>

              {nestedItems.length && (
                <NavAccordionDetails>
                  {nestedItems.map(({ id: nestedId, name, type, distance }, index) => (
                    <div
                      key={index}
                      className="g-flex tab-list__item-card"
                      onPointerUp={() => onNestedItemClick?.(nestedId)}
                    >
                      {type && (
                        <TransportIcon className="g-flex__item-fixed stop__icon" type={type} />
                      )}

                      <div className="g-flex__item">{localizeText(name)}</div>

                      <div className="g-flex__item-fixed distance">{formatDistance(distance)}</div>
                    </div>
                  ))}
                </NavAccordionDetails>
              )}
            </NavAccordion>
          </div>
        ))}
      </TouchScrollWrapper>
    </div>
  );
};
