import cn from "classnames";
import {
  HTMLAttributes,
  ReactNode,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import { LocalizationContext, useServerLocalization } from "@mt/i18n";
import { Order, uuid } from "@mt/common-types";

import { AttractionCard, AccordionListTab, NestedItems } from "./components";
import { Drawer, Icons } from "@mt/components";

import { Attraction, Station } from "./nav-widget.interface";
import { HomeTab } from "./components";
import { StyledNavWidget } from "./nav-widget.styles";

export type NavTabs = "stationsTab" | "attractionsTab";

export interface NavWidgetProps extends HTMLAttributes<HTMLDivElement> {
  stations: Station[];
  attractions: Attraction[];
}
export function NavWidget({ stations, attractions }: NavWidgetProps) {
  const { setLocale, locale } = useContext(LocalizationContext);
  const localizeText = useServerLocalization();
  const [isOpen, setIsOpen] = useState<boolean>(false);
  const [openedTab, setOpenedTab] = useState<NavTabs | null>(null);
  const [attractionId, setAttractionId] = useState<uuid | null>(null);
  const [attractionOrder, setAttractionOrder] = useState<Order>("asc");

  const sortAttractionsBtn: ReactNode = useMemo(() => {
    if (openedTab === "attractionsTab") {
      return (
        <div
          className={cn([
            { "order-btn-inverse": attractionOrder === "desc" },
            "action-btn",
          ])}
          onPointerUp={() =>
            setAttractionOrder((prev) => (prev === "asc" ? "desc" : "asc"))
          }
        >
          <Icons.SortIcon />
        </div>
      );
    }

    return <></>;
  }, [openedTab, setAttractionOrder, attractionOrder]);

  useEffect(() => {
    if (!isOpen) {
      setOpenedTab(null);
    }

    if (!isOpen || !openedTab) {
      setAttractionId(null);
    }
  }, [isOpen, openedTab]);

  const mappedStations = useMemo(() => {
    return (stations ?? []).map(({ nearbyTouristAttractions, ...station }) => ({
      ...station,
      nestedItems: nearbyTouristAttractions as NestedItems,
    }));
  }, [stations]);

  const mappedAttractions = useMemo(() => {
    return (attractions ?? [])
      .map(({ nearbyTransportStops, ...attraction }) => ({
        ...attraction,
        nestedItems: nearbyTransportStops as NestedItems,
      }))
      .sort(
        (a, b) =>
          localizeText(a.name)
            .toLowerCase()
            .localeCompare(localizeText(b.name).toLowerCase()) *
          (attractionOrder === "asc" ? 1 : -1)
      );
  }, [attractions, attractionOrder, locale]);

  const handleExpandChange = (isExpanded: boolean, id: uuid) =>
    setAttractionId(isExpanded ? id : null);

  return (
    <Drawer
      className={cn({ "nav-widget--opened": isOpen })}
      isOpen={isOpen}
      onToggle={setIsOpen}
      onHomeBtnClick={() => setOpenedTab(null)}
      onLocaleChange={setLocale}
      actions={sortAttractionsBtn}
    >
      <StyledNavWidget>
        <HomeTab onOpenTab={setOpenedTab} />

        <AccordionListTab
          titleId="stops"
          isOpened={openedTab === "stationsTab"}
          items={mappedStations}
          onNestedItemClick={setAttractionId}
        />

        <AccordionListTab
          titleId="attractions"
          isOpened={openedTab === "attractionsTab"}
          items={mappedAttractions}
          onExpandChange={handleExpandChange}
        />
      </StyledNavWidget>

      {attractionId && <AttractionCard attractionId={attractionId} />}
    </Drawer>
  );
}
