import { Dispatch, useEffect, useState } from "react";
import { EventQueryData, useEventQuery } from "@mt/utils";
import { TransportType } from "@mt/common-types";
import { LocalizeTextFn, useServerLocalization } from "@mt/i18n";
import { TransferEvent, PublicRoute, TransferItem } from "./transfer.interface";
import { AlertEvent, AlertMessage } from "./alert.interface";

export const useOperativeInfo = () => {
  const localizeText = useServerLocalization();
  const [transfers, setTransfers] = useState<TransferItem[]>([]);
  const [alerts, setAlerts] = useState<AlertMessage[]>([]);

  const { data: events, isSuccess } = useEventQuery<TransferEvent | AlertEvent>(
    "/widgets/operative-info/events",
    ["UPDATE_AVAILABLE_TRANSFERS", "UPDATE_AVAILABLE_ALERTS"]
  );

  useEffect(() => {
    if (isSuccess && events.length) {
      // prettier-ignore
      const transferEvent = events.filter((e) => e['@type'] === 'UPDATE_AVAILABLE_TRANSFERS').at(-1) as EventQueryData<TransferEvent>;
      // prettier-ignore
      const alertEvent = events.filter((e) => e['@type'] === 'UPDATE_AVAILABLE_ALERTS').at(-1) as EventQueryData<AlertEvent>;

      updateTransfers(transferEvent, setTransfers, localizeText);
      updateAlerts(alertEvent, setAlerts);
    }
  }, [events, isSuccess]);

  return { transfers, alerts };
};

function updateTransfers(
  transferEvent: EventQueryData<TransferEvent>,
  setTransfers: Dispatch<TransferItem[]>,
  localizeText: LocalizeTextFn
) {
  if (transferEvent) {
    const { publicRoutes, stations } = transferEvent as TransferEvent;

    const newTransfers: TransferItem[] = [
      ...mapPublicRoutesToTransfers(publicRoutes),
      ...stations.map(({ type, name }) => ({
        type,
        label: localizeText(name),
      })),
    ];

    setTransfers(newTransfers);
  }
}

function updateAlerts(
  alertEvent: EventQueryData<AlertEvent>,
  setAlerts: Dispatch<AlertMessage[]>
) {
  if (alertEvent) {
    const { msgs } = alertEvent as AlertEvent;

    const newAlerts: AlertMessage[] = [...msgs]
      .sort((a, b) => a.order - a.order)
      .map((msg) => ({
        ...msg,
        key: crypto.randomUUID(),
      }));

    setAlerts(newAlerts);
  }
}

function mapPublicRoutesToTransfers(routes: PublicRoute[]): TransferItem[] {
  const routeMap = routes.reduce(
    (acc: Partial<Record<TransportType, string[]>>, item) => {
      const { type, number } = item;

      acc[type] = [...(acc[type] ?? []), number];

      return acc;
    },
    {}
  );

  return Object.entries(routeMap).flatMap(([type, numbers]) => {
    return {
      type,
      label: numbers.join(", "),
    } as TransferItem;
  });
}
